use anyhow::{bail, Result};
use scraper;
use scraper::Element;

fn next_element(element: scraper::ElementRef<'_>) -> Result<scraper::ElementRef<'_>> {
    let next_node = element.next_sibling_element();

    if let Some(node) = next_node {
        return Ok(node);
    }
    bail!("No more elements.");
}

pub fn find_vod(html: &String) -> Result<String> {
    let document = scraper::Html::parse_document(&html);
    // let a_selector = scraper::Selector::parse(r#"div>a[href^="/bible/"]"#).unwrap();
    let mut result: Vec<(String, String)> = Vec::new();

    // How about this?
    {
        let vod_div_select = scraper::Selector::parse("main>div>div>div>div").unwrap();
        if let Some(vod_div) = document.select(&vod_div_select).next() {
            // Ok, search just in this div for things of interest.
            let h1_select = scraper::Selector::parse("h1").unwrap();
            let h1 = vod_div.select(&h1_select).next().unwrap();
            println!("h1 = {:?}", h1.text().collect::<Vec<_>>());
            let p_select = scraper::Selector::parse("p").unwrap();
            let p = vod_div.select(&p_select).next().unwrap();
            println!("p = {:?}", p.text().collect::<Vec<_>>());

            let a_select = scraper::Selector::parse(r#"div[class~="mbs-2"]>a"#).unwrap();
            for a in vod_div.select(&a_select) {
                println!("a = {:?}", a.text().collect::<Vec<_>>());
            }
        }
    }

    /* 
    // Verse of the day is div with two a's.
    let h1_selector = scraper::Selector::parse("div>h1").unwrap();
    let h1 = document.select(&h1_selector).next().unwrap();
    println!("{:?}", h1.text().collect::<Vec<_>>());
    let a = next_element(h1).unwrap();
    println!("a : {}", a.html());
    */

    /*
    let a = next_element(a).unwrap();
    println!("a : {}", a.html());
    */
    // Previous ones are in div[class="mlb-2"]

    let prev_div_selector = scraper::Selector::parse(r#"div[class="mlb-2"]"#).unwrap();
    let a_selector1 =
        scraper::Selector::parse(r#"a[href^="/bible/"][class~="no-underline"]"#).unwrap();
    let p_selector = scraper::Selector::parse("div>p").unwrap();

    println!("=====");
    
    for prev_div in document.select(&prev_div_selector) {
        if let Some(p) = prev_div.select(&p_selector).next() {
            println!("{:?}", p.text().collect::<Vec<_>>());
        }

        let mut last_verse = String::new();
        for a in prev_div.select(&a_selector1) {
            if let Some(href) = a.attr("href") {
                let text = a.text().collect::<Vec<_>>();
                println!("{:?}", text);
                // println!("html: {}", a.html());
            };
        }
        println!("-----");
    }

    /*
    println!("And finally...");
    let a_selector =
        scraper::Selector::parse(r#"div>a[href^="/bible/"][class~="no-underline"]"#).unwrap();
    let mut last_verse = String::new();
    for a in document.select(&a_selector) {
        if let Some(href) = a.attr("href") {
            let text = a.text().collect::<Vec<_>>();
            println!("{:?}", text);
            println!("html: {}", a.html());
        };
    }
    */
    bail!("More dERP!");
}

pub fn find_next_chapter(html: &String) -> Result<String> {
    let document = scraper::Html::parse_document(html);
    // let a_selector = scraper::Selector::parse("div>a").unwrap();
    // This one works:  (starts with "/bible/").
    // let a_selector = scraper::Selector::parse(r#"div>a[href ^= "/bible/"]"#).unwrap();
    // This one fails to find what we're looking for.  Contains /bible/ or "bible" both fail.
    // Ok, using href~="/bible/" fails.  It looks for the WORD "/bible/".
    // Using *= finds it anywhere.  Ah HA!
    let a_selector = scraper::Selector::parse(r#"div>a[href*="/bible/"]"#).unwrap();

    for a in document.select(&a_selector) {
        if a.attr("class").is_some() {
            continue;
        }

        // Since the selector finds href containing /bible/, I don't need some of these tests now.
        // I still need this one, so I have the href value.

        if let Some(href) = a.attr("href") {
            //    if href.contains("/bible/") {

            let text = a
                .text()
                .map(|s| {
                    s.trim_matches(char::is_whitespace) // &[' ', '\n', '\t'])
                })
                .filter(|x| {
                    !x.is_empty()
                    // x.chars().any(|c| (c != ' ') && (c != '\n'))
                })
                .collect::<Vec<_>>();
            // println!("TEXT: {:?}", text);

            if text.len() != 1 {
                continue;
            }
            if text[0] != "Next Chapter" {
                // println!("Found: [{:?}]", text[0]);
                continue;
            }
            return Ok(href.to_string());

            //    } else {
            //        println!("href contains: [{}]", href);
            //    }
        }
    }
    bail!("Next Chapter not found.");
}

#[cfg(test)]
mod tests {
    use super::*;

    /// Test HTML as given to us by the website.
    #[test]
    fn chapter_test() {
        let html = String::from(
            r#"<div class="[pointer-events:all]"><a href="/bible/59/GEN.2.ESV"><div class="flex items-center justify-center bg-white z-[5] h-[50px] w-[50px] rounded-full border-gray-15 border-small border-solid shadow-light-1 active:shadow-dark-1 active:bg-gray-5 active:border-b-gray-5"><svg width="25" height="25" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" aria-labelledby="Next Chapter" class="text-gray-25"><title id="Next Chapter">Next Chapter</title><path fill-rule="evenodd" clip-rule="evenodd" d="M8.293 18.707a1 1 0 0 1 0-1.414l4.94-4.94a.5.5 0 0 0 0-.707l-4.94-4.939a1 1 0 0 1 1.414-1.414l5.647 5.646a1.5 1.5 0 0 1 0 2.122l-5.647 5.646a1 1 0 0 1-1.414 0Z" fill="currentColor"></path></svg></div></a></div>"#,
        );
        let r = find_next_chapter(&html);
        if !r.is_ok() {
            println!("DEBUG result = {:?}", r);
        }
        assert!(r.is_ok());
        let link = r.unwrap();
        assert_eq!(link, "/bible/59/GEN.2.ESV");
    }

    /// This tests when the HTML has been tidied.
    ///
    /// HTML has newlines and spaces added, rather then condensed.
    #[test]
    fn chapter_test_tidy() {
        let html = String::from(
            r#"<div>
        <a href="/bible/59/GEN.2.ESV">
        <div class="flex items-center justify-center bg-white z-[5] h-[50px] w-[50px] rounded-full border-gray-15 border-small border-solid shadow-light-1 active:shadow-dark-1 active:bg-gray-5 active:border-b-gray-5">
        <svg width="25" height="25" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" aria-labelledby="Next Chapter" class="text-gray-25">
        <title id="Next Chapter">Next Chapter</title>
        <path fill-rule="evenodd" clip-rule="evenodd" d="M8.293 18.707a1 1 0 0 1 0-1.414l4.94-4.94a.5.5 0 0 0 0-.707l-4.94-4.939a1 1 0 0 1 1.414-1.414l5.647 5.646a1.5 1.5 0 0 1 0 2.122l-5.647 5.646a1 1 0 0 1-1.414 0Z" fill="currentColor">
        </path>
        </svg>
        </div>
        </a>
        </div>"#,
        );
        let r = find_next_chapter(&html);
        if !r.is_ok() {
            println!("DEBUG result = {:?}", r);
        }
        assert!(r.is_ok());
        let link = r.unwrap();
        assert_eq!(link, "/bible/59/GEN.2.ESV");
    }
}
